"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
Object.defineProperty(exports, "BeaconInfo", {
  enumerable: true,
  get: function () {
    return _types.BeaconInfo;
  }
});
Object.defineProperty(exports, "NotifcationInfo", {
  enumerable: true,
  get: function () {
    return _types.NotifcationInfo;
  }
});
Object.defineProperty(exports, "SipCallState", {
  enumerable: true,
  get: function () {
    return _types.SipCallState;
  }
});
Object.defineProperty(exports, "SipRegistrationState", {
  enumerable: true,
  get: function () {
    return _types.SipRegistrationState;
  }
});
exports.toggleVideo = exports.toggleSpeaker = exports.toggleMute = exports.toggleCamera = exports.stopSipApplication = exports.stopScanService = exports.stopLocatingService = exports.startScanService = exports.startLocatingService = exports.rejectIncomingCall = exports.initSipApplication = exports.getServiceStatus = exports.answerIncomingCall = exports.SipVideoCallPreview = void 0;

var _reactNative = require("react-native");

var _types = require("./types");

var _react = _interopRequireDefault(require("react"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const LINKING_ERROR = "The package 'react-native-jarvis-template-app-sdk' doesn't seem to be linked. Make sure: \n\n" + _reactNative.Platform.select({
  ios: "- You have run 'pod install'\n",
  default: ''
}) + '- You rebuilt the app after installing the package\n' + '- You are not using Expo managed workflow\n';
const JarvisTemplateAppSdk = _reactNative.NativeModules.JarvisTemplateAppSdk ? _reactNative.NativeModules.JarvisTemplateAppSdk : new Proxy({}, {
  get() {
    throw new Error(LINKING_ERROR);
  }

});
let onAccountRegistrationStateListener;
let onCallStateListener;
const eventEmitter = new _reactNative.NativeEventEmitter(_reactNative.NativeModules.JarvisTemplateAppSdk);
let proximityPushCallback;
let gpsLocatingCallback;
eventEmitter.addListener('BeaconInformation', event => {
  const {
    uuid,
    minor,
    major,
    time,
    title,
    description
  } = event;
  const device = {
    uuid,
    minor,
    major
  };
  const notification = {
    title,
    description
  };

  if (proximityPushCallback) {
    proximityPushCallback(device, notification, time);
  }
});
eventEmitter.addListener('GpsInformation', event => {
  const {
    uuid,
    minor,
    major,
    time,
    userId,
    lat,
    lng
  } = event;
  const device = {
    uuid,
    minor,
    major
  };

  if (gpsLocatingCallback) {
    gpsLocatingCallback(device, userId, lat, lng, time);
  }
});
eventEmitter.addListener('SipAppAccountState', event => {
  const {
    state
  } = event;
  !!onAccountRegistrationStateListener && onAccountRegistrationStateListener(state);
});
eventEmitter.addListener('SipCallState', event => {
  const {
    state,
    remoteAddress
  } = event;
  !!onCallStateListener && onCallStateListener(state, remoteAddress);
});

const startScanService = async (sdkKey, locatingRange, notificationIconName, notificationTitle, notificationDescription, onProximityPush) => {
  proximityPushCallback = onProximityPush;
  return await JarvisTemplateAppSdk.startScanService(sdkKey, locatingRange, notificationIconName, notificationTitle, notificationDescription);
};

exports.startScanService = startScanService;

const stopScanService = async () => {
  // @ts-ignore
  proximityPushCallback = null;
  return await JarvisTemplateAppSdk.stopScanService();
};

exports.stopScanService = stopScanService;

const getServiceStatus = async () => {
  return await JarvisTemplateAppSdk.getScanServiceStatus();
};

exports.getServiceStatus = getServiceStatus;

const startLocatingService = async (sdkKey, locatingRange, notificationIconName, notificationTitle, notificationDescription, onGpsFound) => {
  gpsLocatingCallback = onGpsFound;
  return await JarvisTemplateAppSdk.startLocatingService(sdkKey, locatingRange, notificationIconName, notificationTitle, notificationDescription);
};

exports.startLocatingService = startLocatingService;

const stopLocatingService = async () => {
  // @ts-ignore
  gpsLocatingCallback = null;
  return await JarvisTemplateAppSdk.stopLocatingService();
};

exports.stopLocatingService = stopLocatingService;

const initSipApplication = async (username, password, onAccountRegistrationStateChange, onCallStateChange) => {
  onAccountRegistrationStateListener = onAccountRegistrationStateChange;
  onCallStateListener = onCallStateChange;
  return await JarvisTemplateAppSdk.initSipApplication(username, password);
};

exports.initSipApplication = initSipApplication;

const stopSipApplication = () => {
  // @ts-ignore
  onAccountRegistrationStateListener = null; // @ts-ignore

  onCallStateListener = null;
  JarvisTemplateAppSdk.stopSipApplication();
};

exports.stopSipApplication = stopSipApplication;

const answerIncomingCall = () => {
  JarvisTemplateAppSdk.answerIncomingCall();
};

exports.answerIncomingCall = answerIncomingCall;

const rejectIncomingCall = () => {
  JarvisTemplateAppSdk.rejectIncomingCall();
};

exports.rejectIncomingCall = rejectIncomingCall;

const toggleMute = () => {
  JarvisTemplateAppSdk.toggleMute();
};

exports.toggleMute = toggleMute;

const toggleSpeaker = () => {
  JarvisTemplateAppSdk.toggleSpeaker();
};

exports.toggleSpeaker = toggleSpeaker;

const toggleVideo = () => {
  JarvisTemplateAppSdk.toggleVideo();
};

exports.toggleVideo = toggleVideo;

const toggleCamera = () => {
  JarvisTemplateAppSdk.toggleCamera();
};

exports.toggleCamera = toggleCamera;
const SipVideoCallPreviewRaw = (0, _reactNative.requireNativeComponent)('SipVideoCallPreview');

const SipVideoCallPreview = props => {
  return /*#__PURE__*/_react.default.createElement(SipVideoCallPreviewRaw, props);
};

exports.SipVideoCallPreview = SipVideoCallPreview;
//# sourceMappingURL=index.js.map