import { NativeEventEmitter, NativeModules, Platform, requireNativeComponent } from 'react-native';
import { BeaconInfo, NotifcationInfo, SipCallState, SipRegistrationState } from './types';
import React from 'react';
const LINKING_ERROR = "The package 'react-native-jarvis-template-app-sdk' doesn't seem to be linked. Make sure: \n\n" + Platform.select({
  ios: "- You have run 'pod install'\n",
  default: ''
}) + '- You rebuilt the app after installing the package\n' + '- You are not using Expo managed workflow\n';
const JarvisTemplateAppSdk = NativeModules.JarvisTemplateAppSdk ? NativeModules.JarvisTemplateAppSdk : new Proxy({}, {
  get() {
    throw new Error(LINKING_ERROR);
  }

});
let onAccountRegistrationStateListener;
let onCallStateListener;
const eventEmitter = new NativeEventEmitter(NativeModules.JarvisTemplateAppSdk);
let proximityPushCallback;
let gpsLocatingCallback;
eventEmitter.addListener('BeaconInformation', event => {
  const {
    uuid,
    minor,
    major,
    time,
    title,
    description
  } = event;
  const device = {
    uuid,
    minor,
    major
  };
  const notification = {
    title,
    description
  };

  if (proximityPushCallback) {
    proximityPushCallback(device, notification, time);
  }
});
eventEmitter.addListener('GpsInformation', event => {
  const {
    uuid,
    minor,
    major,
    time,
    userId,
    lat,
    lng
  } = event;
  const device = {
    uuid,
    minor,
    major
  };

  if (gpsLocatingCallback) {
    gpsLocatingCallback(device, userId, lat, lng, time);
  }
});
eventEmitter.addListener('SipAppAccountState', event => {
  const {
    state
  } = event;
  !!onAccountRegistrationStateListener && onAccountRegistrationStateListener(state);
});
eventEmitter.addListener('SipCallState', event => {
  const {
    state,
    remoteAddress
  } = event;
  !!onCallStateListener && onCallStateListener(state, remoteAddress);
});

const startScanService = async (sdkKey, locatingRange, notificationIconName, notificationTitle, notificationDescription, onProximityPush) => {
  proximityPushCallback = onProximityPush;
  return await JarvisTemplateAppSdk.startScanService(sdkKey, locatingRange, notificationIconName, notificationTitle, notificationDescription);
};

const stopScanService = async () => {
  // @ts-ignore
  proximityPushCallback = null;
  return await JarvisTemplateAppSdk.stopScanService();
};

const getServiceStatus = async () => {
  return await JarvisTemplateAppSdk.getScanServiceStatus();
};

const startLocatingService = async (sdkKey, locatingRange, notificationIconName, notificationTitle, notificationDescription, onGpsFound) => {
  gpsLocatingCallback = onGpsFound;
  return await JarvisTemplateAppSdk.startLocatingService(sdkKey, locatingRange, notificationIconName, notificationTitle, notificationDescription);
};

const stopLocatingService = async () => {
  // @ts-ignore
  gpsLocatingCallback = null;
  return await JarvisTemplateAppSdk.stopLocatingService();
};

const initSipApplication = async (username, password, onAccountRegistrationStateChange, onCallStateChange) => {
  onAccountRegistrationStateListener = onAccountRegistrationStateChange;
  onCallStateListener = onCallStateChange;
  return await JarvisTemplateAppSdk.initSipApplication(username, password);
};

const stopSipApplication = () => {
  // @ts-ignore
  onAccountRegistrationStateListener = null; // @ts-ignore

  onCallStateListener = null;
  JarvisTemplateAppSdk.stopSipApplication();
};

const answerIncomingCall = () => {
  JarvisTemplateAppSdk.answerIncomingCall();
};

const rejectIncomingCall = () => {
  JarvisTemplateAppSdk.rejectIncomingCall();
};

const toggleMute = () => {
  JarvisTemplateAppSdk.toggleMute();
};

const toggleSpeaker = () => {
  JarvisTemplateAppSdk.toggleSpeaker();
};

const toggleVideo = () => {
  JarvisTemplateAppSdk.toggleVideo();
};

const toggleCamera = () => {
  JarvisTemplateAppSdk.toggleCamera();
};

const SipVideoCallPreviewRaw = requireNativeComponent('SipVideoCallPreview');

const SipVideoCallPreview = props => {
  return /*#__PURE__*/React.createElement(SipVideoCallPreviewRaw, props);
};

export { startScanService, stopScanService, getServiceStatus, startLocatingService, stopLocatingService, initSipApplication, stopSipApplication, answerIncomingCall, rejectIncomingCall, toggleMute, toggleSpeaker, toggleVideo, toggleCamera, BeaconInfo, NotifcationInfo, SipVideoCallPreview, SipCallState, SipRegistrationState };
//# sourceMappingURL=index.js.map